/*
T-SQL-Abfragen fr Microsoft SQL-Server 2022
Kontakt per E-Mail: info@it-angermann.de
Letzte Aktualisierung:  23 Mrz 2024.
Dirk Angermann
https://it-angermann.de
*/

-- Beispielskript zum Erstellen einer Datenbank mit SQL Server 2022
-- Copyright 2024 Dirk Angermann IT-Training & Consulting

USE master;
go

-- eventuell vorhandene Datenbank lschen
DROP DATABASE IF EXISTS db_Test;
go

-- Neue Datenbank 'db_Test' erstellen
CREATE DATABASE db_Test
/*
ON PRIMARY
( 
	NAME = 'db_Test_Data'
	,FILENAME = 'C:\Program Files\Microsoft SQL Server\MSSQL16.PRODUKTION\MSSQL\DATA\db_Test_Data.mdf'
	,SIZE = 10 MB
	,MAXSIZE = 50 MB
	,FILEGROWTH = 10 MB
)

LOG ON
( 
	NAME = 'db_Test_Log'
	,FILENAME = 'C:\Program Files\Microsoft SQL Server\MSSQL16.PRODUKTION\MSSQL\DATA\db_Test_Log.ldf'
	,SIZE = 1 MB
	,MAXSIZE = 25 MB
	,FILEGROWTH = 1 MB
);
*/
go

-- Wechsel in die Datenbank db_Test
USE db_Test;
go

-- Tabellen erstellen
DROP TABLE IF EXISTS dbo.Artikel;
go

CREATE TABLE dbo.Artikel
	(
	ANr VARCHAR(3) NOT NULL
	,AName VARCHAR(20) NOT NULL
	,Farbe VARCHAR(7) NULL
	,Gewicht DECIMAL(9,2) NULL
	,AStadt VARCHAR(20) NULL
	,AMenge INT NULL
	,CONSTRAINT PK_ANr PRIMARY KEY(ANr)
	,CONSTRAINT C_ANr CHECK(ANr LIKE 'A%' AND CAST(SUBSTRING(ANr,2,2) AS INT) BETWEEN 1 AND 99)
	,CONSTRAINT C_AStadt CHECK(AStadt LIKE '[A-Z]%')
	,CONSTRAINT C_Farbe CHECK(Farbe IN ('rot','blau','grn','schwarz','gelb'))
	,CONSTRAINT C_AMenge CHECK(AMenge BETWEEN 0 AND 10000)
	);
go

DROP TABLE IF EXISTS dbo.Lieferant;
go

CREATE TABLE dbo.Lieferant
	(
	LNr VARCHAR(3) NOT NULL  
	,LName VARCHAR(20) NOT NULL
	,Status INT NOT NULL CONSTRAINT DF_Zahl DEFAULT(0)
	,LStadt VARCHAR(20) NULL
	,CONSTRAINT PK_LNr PRIMARY KEY(LNr)
	,CONSTRAINT C_LNr CHECK(LNr LIKE 'L%' AND CAST(SUBSTRING(LNr,2,2) AS INT) BETWEEN 1 AND 99)
	,CONSTRAINT C_LName CHECK(LName LIKE '[A-Z]%')
	,CONSTRAINT C_LStadt CHECK(LStadt LIKE '[A-Z]%')
	);
go

DROP TABLE IF EXISTS dbo.Lieferung;
go

CREATE TABLE dbo.Lieferung
	(
	LNr VARCHAR(3) NOT NULL
	,ANr VARCHAR(3) NOT NULL
	,LMenge INT NOT NULL
	,LDatum DATE NOT NULL
	,CONSTRAINT ZPK_Lief PRIMARY KEY(LNr,ANr,LDatum)
	,CONSTRAINT FK_LNr FOREIGN KEY(LNr) REFERENCES dbo.Lieferant(LNr)
	,CONSTRAINT FK_ANr FOREIGN KEY(ANr) REFERENCES dbo.Artikel(ANr)
	,CONSTRAINT C_LMenge CHECK(LMenge BETWEEN 0 AND 10000)
	);
go

-- Trigger erstellen
DROP TRIGGER IF EXISTS dbo.tr_INSERT;
go

CREATE TRIGGER dbo.tr_INSERT
ON dbo.Lieferung
AFTER INSERT
AS
UPDATE dbo.Lieferant
SET Status = Status + 1
FROM dbo.Lieferant AS a INNER JOIN INSERTED AS b ON a.LNr = b.LNr;

UPDATE dbo.Artikel
SET AMenge = AMenge + b.LMenge
FROM dbo.Artikel AS a INNER JOIN INSERTED AS b ON a.ANr = b.ANr; 
go

DROP TRIGGER IF EXISTS dbo.tr_DELETE;
go

CREATE TRIGGER dbo.tr_DELETE
ON dbo.Lieferung
AFTER DELETE
AS
DECLARE @Cur_LNr VARCHAR(5)
		,@Cur_ANr VARCHAR(5)
		,@cur_LMenge INT;

DECLARE cur_Lief_Lsch CURSOR
FOR
SELECT
	LNr
	,ANr
	,LMenge
FROM DELETED;
    
OPEN cur_Lief_Lsch;

FETCH NEXT FROM cur_Lief_Lsch INTO @cur_LNr, @cur_ANr, @cur_LMenge;

WHILE @@FETCH_STATUS = 0
BEGIN
	UPDATE dbo.Artikel
	SET AMenge = AMenge - @cur_LMenge
	WHERE ANr = @cur_ANr

	UPDATE dbo.Lieferant
	SET Status = Status - 1
	WHERE LNr = @cur_LNr

	FETCH NEXT FROM cur_lief_lsch INTO @cur_LNr, @cur_ANr, @cur_LMenge
END;
CLOSE cur_Lief_Lsch;
DEALLOCATE cur_Lief_Lsch;
go

DROP TRIGGER IF EXISTS dbo.tr_UPDATE;
go

CREATE TRIGGER dbo.tr_UPDATE
ON dbo.Lieferung
AFTER UPDATE
AS
IF UPDATE (LMenge)
BEGIN
   UPDATE dbo.Artikel
   SET AMenge = AMenge + (SELECT SUM(LMenge) FROM INSERTED) 
					   - (SELECT SUM(LMenge) FROM DELETED)
   FROM dbo.Artikel AS a INNER JOIN INSERTED AS b ON a.ANr = b.ANr 
						 INNER JOIN DELETED AS c ON a.ANr = c.ANr
END;
go

SET NOCOUNT ON;
go

--Tabellen mit Anfangsdaten fllen
INSERT INTO dbo.Lieferant VALUES('L01','Schmidt',0,'Hamburg'),
		      			 		('L02','Jonas',0,'Ludwigshafen'),
							    ('L03','Blank',0,'Ludwigshafen'),
							    ('L04','Clark',0,'Hamburg'),
							    ('L05','Adam',0,'Aachen');
go

INSERT INTO dbo.Artikel VALUES('A01','Mutter','rot',12,'Hamburg',0),
						      ('A02','Bolzen','grn',17,'Ludwigshafen',0),
						      ('A03','Schraube','blau',17,'Mannheim',0),
						      ('A04','Schraube','rot',14,'Hamburg',0),
						      ('A05','Nockenwelle','blau',12,'Ludwigshafen',0),
						      ('A06','Zahnrad','rot',19,'Hamburg',0);
go

INSERT INTO dbo.Lieferung VALUES('L01','A01',300,'20190518');
INSERT INTO dbo.Lieferung VALUES('L01','A02',200,'20190713');
INSERT INTO dbo.Lieferung VALUES('L01','A03',400,'20190101');
INSERT INTO dbo.Lieferung VALUES('L01','A04',200,'20190725');
INSERT INTO dbo.Lieferung VALUES('L01','A05',100,'20190801');
INSERT INTO dbo.Lieferung VALUES('L01','A06',100,'20190723');
INSERT INTO dbo.Lieferung VALUES('L02','A01',300,'20190802');
INSERT INTO dbo.Lieferung VALUES('L02','A02',400,'20190805');
INSERT INTO dbo.Lieferung VALUES('L03','A02',200,'20190806');
INSERT INTO dbo.Lieferung VALUES('L04','A02',200,'20190809');
INSERT INTO dbo.Lieferung VALUES('L04','A04',300,'20190820');
INSERT INTO dbo.Lieferung VALUES('L04','A05',400,'20190821');
go

SET NOCOUNT OFF;
go

PRINT '';
PRINT 'Die Datenbank "' + DB_NAME()+ '" wurde angelegt am ' 
					    + CONVERT(VARCHAR(10),SYSDATETIME(),104)
					    + ' um '
					    + CONVERT(VARCHAR(10),SYSDATETIME(),108)
					    +'.';
go

SET NOCOUNT ON;
go

-- Abfragen der Tabellen
SELECT 
	LNr AS 'Lieferantennummer'
	,LName AS 'Lieferantenname'
	,Status AS 'Status des Lieferanten'
	,LStadt AS 'Firmensitz'
FROM dbo.Lieferant;
go

SELECT
	Anr AS 'Artikelnummer'
	,AName AS 'Artikelname'
	,Farbe
	,Gewicht
	,AStadt AS 'Lagerort'
	,AMenge AS 'Lagermenge'
FROM dbo.Artikel;
go

SELECT 
	LNr AS 'Lieferantennummer'
	,ANr AS 'Artikelnummer'
	,LMenge AS 'Gelieferte Menge'
	,CONVERT(VARCHAR(10),LDatum,104) AS 'Lieferdatum'
FROM dbo.Lieferung;
go

SET NOCOUNT OFF;
go