/*
T-SQL-Abfragen fr Microsoft SQL-Server 2022
Kontakt per E-Mail: info@it-angermann.de
Letzte Aktualisierung:  23 Mrz 2024.
Dirk Angermann
https://it-angermann.de
*/

-- Ausgabe der Zeichenkette  --> abc
-- Alphanumerische- und Datumsformate immer in Hochkomma
SELECT 'abc';

-- Verbinden von Zeichenketten	
SELECT 'Heute  ' + 'erlerne ich '  + 'die Grundlagen ' + 'von T-SQL.';
	
-- Berechnungen in SELECT
-- numerische Zeichen nicht in Hochkomma
SELECT 133 * 22;		

-- Ausgabe von globalen Variablen
-- Versionsnummer	
SELECT @@VERSION;

-- Instanzname
SELECT @@SERVERNAME;

-- Ausgabe von Funktionen
-- Datum/Uhrzeit
SELECT GETDATE();
SELECT SYSDATETIME();

-- aktueller Datenbankname	
SELECT DB_NAME();

-- aktuelle angemeldeter Benutzer
SELECT SUSER_SNAME();


USE db_Test;
go

-- Gesucht sind die Spalten Artikelnummer und Artikelname der Tabelle Artikel.
SELECT 
	ANr
	,AName
FROM dbo.Artikel;
go

-- Gesucht sind alle Angaben der Tabelle Artikel.
SELECT *
FROM dbo.Artikel;
go

-- Spalten in einer vernderten Reihenfolge, als die Tabellendefinition
SELECT 
	ANr
	,AMenge
	,Farbe
	,AStadt
	,Gewicht
	,AName
FROM dbo.Artikel;
go

-- Gesucht sind der Artikelname und das dazugehrige Gewicht in 
-- Kilogramm.

SELECT
	AName
	,Gewicht/1000
FROM dbo.Artikel;
go

-- Gesucht sind der Artikelname und das dazugehrige Gesamtgewicht in 
-- Kilogramm.
SELECT 
	AName
	,Gewicht * AMenge/1000
FROM dbo.Artikel;
go

-- Gesucht sind der Artikelname und das dazugehrige Gewicht in Kilogramm
-- Das Gewicht soll auf zwei Stellen nach dem Komma gerundet sein.
SELECT 
	AName
	,ROUND(Gewicht/1000,2)
FROM dbo.Artikel;
go

-- Gesucht sind alle Lieferantennummern der Tabelle Lieferung, welche
-- geliefert haben.
-- Problem: 12 Datenstze, welche evtl. doppelt vorkommen
SELECT 
	LNr
FROM dbo.Lieferung;
go

-- Lsung !!! DISTINCT
SELECT DISTINCT 
	LNr
FROM dbo.Lieferung;
go

-- Verwendung von einem Spaltenalias
SELECT 
	AName AS Artikelname
	,AMenge AS Lagermenge
FROM dbo.Artikel;

-- Anmerkung: Sptestens, wenn ein Leerzeichen in der Spaltenberschrift
-- ist, mssen Hochkommas genommen werden.

-- Beispiel 2:
SELECT 
	AName 'Artikelname'
	,AMenge 'Lagermenge'
FROM dbo.Artikel;

-- Beispiel 3:
SELECT 
	AName AS 'Artikelname'
	,AMenge AS 'Lagermenge'
FROM dbo.Artikel;

-- Beispiel 4
SELECT 
	AName AS "Artikelname"
	,AMenge AS "Lagermenge"
FROM dbo.Artikel;

-- Beispiel 5 (Abgekndigt)
SELECT 
	'Artikelname' = AName 
	,'Lagermenge' = AMenge 
FROM dbo.Artikel;
go

-- Anhngen einer zustzlichen Spalte in der Stck angezeigt werden
-- soll
SELECT 
	AName AS 'Artikelname'
	,AMenge AS 'Lagermenge'
	,'Stck' AS 'Bezeichnung'
FROM Artikel;
go

-- Vergleich eines Spaltenwertes lediglich auf genaue bereinstimmung mit
-- einem vorgegebenen Wert.

SELECT  
	'Lieferantennummer' = CASE LNr
					WHEN 'L01' THEN 'Lieferant 1'
					WHEN 'L02' THEN 'Lieferant 2'
					WHEN 'L03' THEN 'Lieferant 3'
					WHEN 'L04' THEN 'Lieferant 4'
					WHEN 'L05' THEN 'Lieferant 5'
					ELSE 'Unbekannt'
				  END
	,LNr AS 'Abkrzung' 
FROM dbo.Lieferant;
go

-- Nun flexibler in der Ausgabe durch CASE.
SELECT 
	AName AS 'Artikelname'
	,AMenge AS 'Lagermenge'
	,'Bezeichnung' = CASE AName
						WHEN 'Mutter' THEN 'Schachtel'
						WHEN 'Bolzen' THEN 'Holzkiste'
						WHEN 'Schraube' THEN 'Tte'
						WHEN 'Nockenwelle' THEN 'Container'
						ELSE 'Unbekannt'
				    END
FROM dbo.Artikel;
go