/*
T-SQL-Abfragen fr Microsoft SQL-Server 2022
Kontakt per E-Mail: info@it-angermann.de
Letzte Aktualisierung:  23 Mrz 2024.
Dirk Angermann
https://it-angermann.de
*/

USE db_Test;
go

---**********************************************************************
-- * VERGLEICHSOPERATOREN -- werden bentigt, um Spalteninhalte mit     *
---* konstanten oder ermittelten Werten zu vergleichen                  *
-- **********************************************************************

-- Gesucht sind die Namen der Lieferanten, die in Hamburg wohnen.
SELECT 
	LName AS 'Name des Lieferanten'
FROM dbo.Lieferant
WHERE LStadt = 'Hamburg';
go 

-- Gesucht sind alle Lieferungen, die vor dem heutigen Zeitpunkt
-- stattgefunden haben.
SELECT
	LNr AS 'Nummer des Lieferanten'
	,ANr AS 'Artikelnummer'
	,LMenge AS 'gelieferte Menge'
	,LDatum	AS 'Lieferdatum'
FROM dbo.Lieferung
WHERE LDatum < GETDATE();
go

-- **********************************************************************
-- * BEREICHE -- Angabe von Grenzen                                     * 
-- * Diese sind immer von  bis (einschlielich)                        *                                                 
-- **********************************************************************

-- Gesucht sind die Lieferanten, die vom 01.07.2019 bis 06.08.2019
-- geliefert haben.
SELECT DISTINCT 
	LNr AS 'Lieferantennummer'
FROM dbo.Lieferung
WHERE LDatum BETWEEN '20190701' AND '20190806';
go

-- **********************************************************************
-- * LISTEN -- Vergleich mit mehr als einen Wert                        *
-- **********************************************************************

-- Gesucht sind alle Lieferanten aus Hamburg, Weimar, Ludwigshafen und 
-- Erfurt!
SELECT
	LName AS 'Name'
	,LStadt AS 'Firmensitz'
FROM dbo.Lieferant
WHERE LStadt IN ('Hamburg','Weimar','Ludwigshafen','Erfurt');
go

-- **********************************************************************
-- * ZEICHENFOLGENBEREINSTIMMUNGEN -- Suche nach bestimmten Teilen von * 
-- * von Zeichenketten                                                  *                                                   
-- **********************************************************************
-- kann nur auf Spalten mit alphanumerischen Werten angewandt werden
-- (CHAR, VARCHAR, ...)
-- Platzhalter:  %  beliebig viele Zeichen (kann auch nichts sein)
-- Platzhalter: _   ein beliebiges Zeichen
-- Platzhalter: [ ] ein Zeichen aus der angegebenen Menge bzw. Bereich

-- Gesucht werden die Namen der Lieferanten, die in einer Stadt wohnen,
-- deren Namen mit L beginnt.
SELECT 
	LName AS 'Name'
	,LStadt AS 'Firmensitz'
FROM dbo.Lieferant
WHERE LStadt LIKE 'L%';
go

-- Gesucht werden die Namen der Lieferanten, die in einer Stadt wohnen,
--  deren Name mit A-L beginnt.
SELECT
	LName AS 'Name'
	,LStadt AS 'Firmensitz'
FROM dbo.Lieferant
WHERE LStadt LIKE '[A-L]%';	-- Beachte A bis L!
go

-- Gesucht werden die Namen der Lieferanten, die in einer Stadt wohnen,
-- deren Name mit A oder L beginnt!
SELECT
	LName AS 'Name'
	,LStadt AS 'Firmensitz'
FROM dbo.Lieferant
WHERE LStadt LIKE '[AL]%';	-- Beachte A oder L!
go

-- **********************************************************************
-- * Verknpfungsoperatoren  -- Verknpfen von mehreren Suchargumenten  *
-- * mit AND und OR                                                     *
-- **********************************************************************
-- AND - zum Abrufen von Zeilen, die alle Suchkriterien erfllen
-- OR  - zum Abrufen von Zeilen, die eines der Suchkriterien erfllen

-- Gesucht werden alle Lieferungen zwischen dem 18.05.2019 und
-- 24.07.2019.
SELECT
	LNr AS 'Lieferantennummer'
	,ANr AS 'Artikelnummer'
	,LMenge AS 'gelieferte Menge'
	,LDatum AS 'Lieferdatum'
FROM dbo.Lieferung
WHERE LDatum >= '20190518' AND LDatum <= '20190724';
go

-- Gesucht werden alle Artikel, deren Farbe rot, gelb oder blau ist.
SELECT
	AName AS 'Artikelname'
	,Farbe
FROM dbo.Artikel
WHERE Farbe = 'rot' OR Farbe = 'gelb' OR Farbe = 'blau';
go

-- **********************************************************************
-- * Unbekannte Werte -- Es werden Werte ermittelt, die NULL  Marken   * -- * enthalten                                                          *
-- **********************************************************************

-- Gesucht werden die Lieferanten, dessen Firmensitz einen NULL-Wert
-- besitzt.
-- Vorbereitung: Datensatz hinzufgen

INSERT INTO dbo.Lieferant VALUES ('L06','Angermann',0,NULL);

-- Abfrage
SELECT
	LName AS 'Name'
	,LStadt AS 'Firmensitz'
FROM dbo.Lieferant
WHERE LStadt IS NULL;
go
