/*
T-SQL-Abfragen fr Microsoft SQL-Server 2022
Kontakt per E-Mail: info@it-angermann.de
Letzte Aktualisierung:  23 Mrz 2024.
Dirk Angermann
https://it-angermann.de
*/

USE db_Test;
go

-- Gesucht sind die Namen der Lieferanten, deren Statuswert grer ist
-- als der durchschnittliche Statuswert aller Lieferanten.
SELECT
	LName AS 'Name'				 -- (zweite Abarbeitung)
	,[Status]
FROM dbo.Lieferant
WHERE Status > (SELECT AVG(Status)
		    	FROM dbo.Lieferant);-- (erste Abarbeitung)


-- Anmerkung: Htten man den durchschnittlichen Statuswert gewusst,
--            htten man gleich folgendes eingeben knnen. 
SELECT
	LName AS 'Name'	
FROM dbo.Lieferant
WHERE Status > 2;

-- Gesucht sind die Nummern aller Lieferanten, die mindestens einen
-- Artikel geliefert haben, den auch Lieferant L03 geliefert hat.
SELECT 
	LNr	AS 'Nummer'			-- (zweite Abarbeitung)
FROM dbo.Lieferung
WHERE ANr = (SELECT ANr		-- (erste Abarbeitung)
	         FROM dbo.Lieferung
	         WHERE LNr = 'L03')
AND LNr <> 'L03';


-- Htten man die Angaben gewusst, htten man gleich folgendes schreiben
-- knnen: 
SELECT 
	LNr	AS 'Nummer'				
FROM dbo.Lieferung
WHERE ANr = 'A02' AND LNr <> 'L03';


-- Gesucht sind die Namen der Lieferanten, die bereits geliefert haben.
SELECT 
	LName AS 'Name'
FROM dbo.Lieferant		
WHERE LNr = (SELECT LNr FROM dbo.Lieferung);


SELECT 
	LName AS 'Name'
FROM dbo.Lieferant		
WHERE LNr IN (SELECT LNr FROM dbo.Lieferung);


-- Gesucht sind die Daten aller Lieferungen von Lieferanten aus Hamburg.
SELECT
	LNr AS 'Lieferantennummer'
	,ANr AS 'Artikelnummer'
	,LMenge AS 'Liefermenge'
	,LDatum AS 'Lieferdatum'
FROM dbo.Lieferung
WHERE LNr IN (SELECT LNr
			  FROM dbo.Lieferant
			  WHERE LStadt = 'Hamburg');


-- Gesucht sind die Nummern und Namen der Artikel, die von Schmidt
-- geliefert wurden.
-- Anmerkung: Verschachtelungen bis auf 32 Ebenen mglich
SELECT 
	ANr AS 'Nummer'
	,Aname AS 'Name'
FROM dbo.Artikel
WHERE ANr IN (SELECT ANr
			 FROM dbo.Lieferung
			 WHERE LNr IN (SELECT LNr
						  FROM dbo.Lieferant
						  WHERE LName = 'Schmidt'));


-- Gesucht sind die Lieferdaten seit dem 01.07.2019, die von Hamburger 
-- Lieferanten durchgefhrt wurden und bei denen rote Artikel mit einem
-- Gewicht ber 14 Gramm geliefert wurden.

SELECT
	LNr AS 'Lieferantennummer'
	,ANr AS 'Artikelnummer'
	,LMenge AS 'Liefermenge'
	,CONVERT(CHAR(10),LDatum,104) AS 'Lieferdatum'
FROM dbo.Lieferung
WHERE LDatum >= '20190701'
AND LNr IN (SELECT LNr
		    FROM dbo.Lieferant
	        WHERE LStadt = 'Hamburg')
AND ANr IN (SELECT ANr
		    FROM dbo.Artikel
		    WHERE Farbe = 'rot' AND Gewicht > 14);

-- Gesucht sind alle Artikelnummern deren letzte Lieferung an dem Tag 
-- erfolgt ist, an dem auch Artikel 'A02' zuletzt geliefert worden ist.
-- Anmerkung: Die Abfrage liefert auf Grund der Daten kein Ergebnis.
SELECT
	ANr AS 'Artikelnummer'
FROM dbo.Lieferung
WHERE ANr <> 'A02'
GROUP BY ANr
HAVING MAX(LDatum) = (SELECT MAX(LDatum)
					  FROM dbo.Lieferung
					  WHERE ANr = 'A02');

-- Gesucht sind die Daten der Lieferanten, deren Status mindestens dem
-- grten Statuswert der Stadt entspricht, in welcher der Lieferant 
-- 'L03' wohnt.
SELECT
	LNr AS 'Nummer Lieferant'
	,LName AS 'Name Liefernat'
	,[Status]
	,LStadt AS 'Firmensitz'
FROM dbo.Lieferant
WHERE [Status] >= (SELECT MAX([Status])
				   FROM dbo.Lieferant
				   WHERE LStadt IN (SELECT LStadt
								    FROM dbo.Lieferant
					 			    WHERE LNr = 'L03'));

-- Gesucht sind die Namen der Lieferanten, die Artikel A02 geliefert
-- haben.

SELECT
	LName AS 'Name'
FROM dbo.Lieferant AS a
WHERE 'A02' IN (SELECT ANr
			    FROM dbo.Lieferung AS b
			    WHERE a.LNr = b.LNr);

-- Gesucht sind die Lieferanten, die mehr als zweimal geliefert haben.

SELECT
	LName AS 'Name'
FROM dbo.Lieferant AS a 
WHERE 2 < (SELECT COUNT(*) 
		   FROM dbo.Lieferung AS b 
		   WHERE a.LNr = b.LNr);

-- Gesucht sind die Namen und Lagerorte aller Artikel, wo mehrere Artikel 
-- am gleichen_Ort lagern.

SELECT
	a.AName AS 'Name'
	,a.AStadt AS 'Lagerort'
FROM dbo.Artikel a
WHERE 1 < (SELECT COUNT(*)
		   FROM dbo.Artikel b
		   WHERE a.AStadt = b.AStadt)
ORDER BY a.AStadt;


-- Gesucht sind die Lieferanten, die bereits geliefert haben.
SELECT
	LNr AS 'Nummer'
FROM dbo.Lieferant AS a
WHERE EXISTS (SELECT	LNr
			 FROM dbo.Lieferung AS b
			 WHERE a.LNr = b.LNr);

-- Gesucht sind die Lieferanten, die noch nie geliefert haben
SELECT
	LNr AS 'Nummer'
FROM dbo.Lieferant AS a
WHERE NOT EXISTS (SELECT LNr
				  FROM dbo.Lieferung AS b
				  WHERE a.LNr = b.LNr);

-- Gesucht sind Artikel, die an Orten lagern, in denen auch Lieferanten
-- wohnen.
SELECT
	AName AS 'Name'
FROM dbo.Artikel
WHERE EXISTS (SELECT *
			  FROM dbo.Lieferant
			  WHERE LStadt = AStadt);

-- Welcher Lieferant wohnt an einem Ort, am dem noch andere Lieferanten
-- wohnen?
SELECT
	LNr AS 'Nummer'
	,LName AS 'Name'
	,LStadt AS 'Firmensitz'
FROM dbo.Lieferant AS a
WHERE EXISTS (SELECT LNr
			  FROM dbo.Lieferant AS b
			  WHERE a.LStadt = b.LStadt AND a.LNr <> b.LNr);
