/*
T-SQL-Abfragen fr Microsoft SQL-Server 2022
Kontakt per E-Mail: info@it-angermann.de
Letzte Aktualisierung:  23 Mrz 2024.
Dirk Angermann
https://it-angermann.de
*/

USE db_Test;
go

-- Gesucht sind die Namen der Lieferanten, die bereits geliefert haben
-- Hier Verknpfung ber Schlsselfelder (Primr- und Fremdschlssel)
SELECT DISTINCT
LName AS 'Name'
FROM dbo.Lieferant INNER JOIN dbo.Lieferung 
ON Lieferant.LNr = Lieferung.LNr;

-- Gesucht sind die Namen von Lieferanten, die in einer Stadt wohnen, wo 
-- auch Artikel lagern.
-- Hier Verknpfung ber Spalten, die gleich sind.
SELECT DISTINCT
LName AS 'Name'
FROM dbo.Lieferant INNER JOIN dbo.Artikel 
ON Lieferant.LStadt = Artikel.AStadt;

-- Gesucht sind die Lieferdaten seit dem 01.07.2019, die von Hamburger 
-- Lieferanten durchgefhrt wurden und bei denen rote Artikel mit einem
-- Gewicht ber 14 Gramm geliefert wurden.
SELECT
	LNr AS 'Lieferantennummer'
	,ANr AS 'Artikelnummer'
	,LMenge AS 'Liefermenge'
	,CONVERT(CHAR(10),LDatum,104) AS 'Lieferdatum'
FROM dbo.Lieferung
WHERE LDatum >= '20190701'
AND LNr IN (SELECT LNr
			FROM dbo.Lieferant
			WHERE LStadt = 'Hamburg')
AND ANr IN (SELECT ANr
			FROM dbo.Artikel
			WHERE Farbe = 'rot' AND Gewicht > 14);

SELECT
	Lieferung.LNr AS 'Lieferantennummer'
	,LName AS 'Name Lieferant'
	,Lieferung.ANr AS 'Artikelnummer'
	,AName AS 'Name Artikel'
	,LMenge AS 'Liefermenge'
	,CONVERT(CHAR(10),LDatum,104) AS 'Lieferdatum'
FROM dbo.Lieferant INNER JOIN dbo.Lieferung ON Lieferant.LNr = Lieferung.LNr 
				   INNER JOIN dbo.Artikel ON Artikel.ANr = Lieferung.ANr
WHERE LDatum >= '20190701'
AND LStadt = 'Hamburg'
AND Farbe = 'rot' 
AND Gewicht > 14;

-- Welcher Lieferant hat Bolzen geliefert?
SELECT DISTINCT 
	LName AS 'Name'
FROM dbo.Lieferant AS a INNER JOIN dbo.Lieferung AS c ON a.LNr = c.LNr 
 INNER JOIN dbo.Artikel AS b ON b.ANr = c.ANr 
WHERE AName = 'Bolzen';

-- Gesucht sind die Lieferdaten der Lieferungen (LName, Farbe, LDatum) von 
-- Schmidt und Clark, die rote und blaue Artikel nach dem 01.08.2019 geliefert haben.
SELECT
	LName AS 'Lieferant'
	,Farbe
	,CONVERT(CHAR(10),LDatum,104) AS 'Lieferdatum'
FROM dbo.Lieferant AS a INNER JOIN dbo.Lieferung AS c ON a.LNr = c.LNr
						INNER JOIN dbo.Artikel AS b ON b.ANr = c.ANr
WHERE LName IN ('Schmidt','Clark') 
AND Farbe IN ('rot','blau')
AND LDatum > '20190801';

-- Wer hat wie viele Schrauben geliefert?
SELECT
	LName AS 'Lieferant'
	,SUM(LMenge) AS 'Summe'
FROM dbo.Lieferant AS a INNER JOIN dbo.Lieferung AS c ON a.LNr = c.LNr
 INNER JOIN dbo.Artikel AS b ON b.ANr = c.ANr
WHERE AName = 'Schraube'
GROUP BY LName;

-- Wann hat Lieferant Schmidt zuletzt Nockenwellen geliefert?
SELECT
	LName AS 'Lieferant'
	,CONVERT(CHAR(10),MAX(LDatum),104)) AS 'Letzte Lieferung'
FROM dbo.Lieferant AS a INNER JOIN dbo.Lieferung AS c ON a.LNr = c.LNr
						INNER JOIN dbo.Artikel AS b ON b.ANr = c.ANr
WHERE LName = 'Schmidt' AND AName = 'Nockenwelle'
GROUP BY LName;

-- Den Namen der Lieferanten, die mehr als viermal geliefert haben.
SELECT
	LName AS 'Lieferant'
	,COUNT(ANr) AS 'Wie oft?'
FROM dbo.Lieferant AS a INNER JOIN dbo.Lieferung AS c ON a.LNr = c.LNr
GROUP BY LName
HAVING COUNT(ANr) > 4;

-- Gesucht sind die Nummern und Namen der Lieferanten, an deren Wohnort 
-- mindestens 3 Sorten von Artikeln lagern!
SELECT
	a.LNr AS 'Nummer'
	,LName AS 'Name'
FROM dbo.Lieferant AS a INNER JOIN dbo.Lieferung AS c ON a.LNr = c.LNr
						INNER JOIN dbo.Artikel AS b ON b.ANr = c.ANr
GROUP BY a.LNr, LName
HAVING COUNT(c.ANr) >= 3;

-- Wer hat mehr als einmal Schrauben geliefert?
SELECT
	LName AS 'Lieferant'
	,COUNT(c.ANr) AS 'Wie viele?'
FROM dbo.Lieferant AS a INNER JOIN dbo.Lieferung AS c ON a.LNr = c.LNr
						INNER JOIN dbo.Artikel AS b ON b.ANr = c.ANr
WHERE AName = 'Schraube'
GROUP BY LName
HAVING COUNT(c.ANr) > 1;

-- Gesucht sind die Nummern und Namen der Lieferanten, deren kleinste 
-- Liefermenge ber den durchschnittlichen Liefermengen aller Lieferanten 
-- liegt, die rote Artikel geliefert haben.
-- Frage: Was ist denn die durchschnittliche Liefermenge aller 
--        gelieferten roten Artikel?
SELECT
	 a.LNr AS 'Nummer'
	 ,LName AS 'Name'
FROM dbo.Lieferant AS a INNER JOIN dbo.Lieferung AS c ON a.LNr = c.LNr
GROUP BY a.LNr, LName
HAVING MIN(LMenge) > (SELECT AVG(LMenge)
					  FROM dbo.Artikel AS b INNER JOIN dbo.Lieferung AS c ON b.ANr = c.ANr 
					  WHERE Farbe = 'rot');

-- Gesucht sind alle Angaben zu Lieferanten, die wenigstens genauso oft
-- den Artikel_Bolzen geliefert haben wie Lieferant L01.
SELECT 
	a.LNr AS 'Nummer'
	,LName AS 'Name'
	,Status
	,LStadt AS 'Firmensitz'
	,COUNT(c.ANr) AS 'Anzahl Bolzen'
FROM dbo.Lieferant AS a INNER JOIN dbo.Lieferung AS c ON a.LNr = c.LNr
						INNER JOIN dbo.Artikel AS b ON b.ANr = c.ANr
WHERE AName ='Bolzen' AND c.LNr <> 'L01'
GROUP BY a.LNr, LName, Status, LStadt
HAVING COUNT(c.ANr) >= (SELECT COUNT(*)
						FROM dbo.Lieferung AS c INNER JOIN dbo.Artikel AS b ON c.ANr = b.ANr
						WHERE AName ='Bolzen' AND c.LNr ='L01');

-- Gesucht sind alle Angaben ber Artikel, die in einer Stadt lagern, 
-- in der auch noch andere Artikel lagern.
SELECT DISTINCT
	a.ANr AS 'Nummer'
	,a.AName AS 'Name'
	,a.Farbe
	,a.Gewicht
	,a.AStadt AS 'Lagerort'
	,a.AMenge AS 'Lagermenge'
FROM dbo.Artikel AS a INNER JOIN dbo.Artikel AS b ON a.AStadt = b.AStadt
WHERE a.ANr <> b.ANr;

-- Gesucht sind alle Lieferanten mit ihren Lieferungen, aber auch die
-- Lieferanten, die noch nie geliefert haben.
-- LEFT OUTER JOIN
SELECT
	a.LNr AS 'Nummer'
	,LName AS 'Name'
	,ANr AS 'Artikelnummer'
	,LMenge AS 'Gelieferte Menge'
	,CONVERT(CHAR(10),LDatum,104) AS 'Lieferdatum'
FROM dbo.Lieferant AS a LEFT OUTER JOIN dbo.Lieferung AS c ON a.LNr = c.LNr;
-- Anmerkung: Der Abfrageoptimierer stellt fest, dass es in der linken
-- Tabelle eine LNr gibt, aber in der rechten nicht, und gibt
-- diesen zustzlichen Datensatz aus.

-- RIGHT OUTER JOIN
SELECT
	a.LNr AS 'Numme'
	,LName AS 'Name'
	,ANr AS 'Artikelnummer'
	,LMenge AS 'Gelieferte Menge'
	,CONVERT(CHAR(10),LDatum,104) AS 'Lieferdatum'
FROM dbo.Lieferung AS c RIGHT OUTER JOIN dbo.Lieferant AS a ON c.LNr = a.LNr;
-- Anmerkung: Der Abfrageoptimierer stellt fest, dass es in der rechten
-- Tabelle eine LNr gibt, aber in der linken nicht, und gibt 

-- FULL OUTER JOIN
SELECT
	a.LNr AS 'Nummer Lieferant'
	,LName AS 'Name Lieferant'
	,ANr AS 'Artikelnummer'
	,LMenge AS 'Gelieferte Menge'
	,CONVERT(CHAR(10),LDatum,104) AS 'Lieferdatum'
FROM dbo.Lieferant AS a FULL OUTER JOIN dbo.Lieferung AS c ON a.LNr = c.LNr;
-- Anmerkung: Der Abfrageoptimierer stellt fest, dass es entweder in der 
-- linken und/oder rechten Tabelle eine LNr gibt, aber in der
-- linken und/oder rechten nicht, und gibt diesen zustzlichen
-- Datensatz aus.

-- Problem: Zu viele Datenstze, die eventuell nicht von Interesse 
-- sind.
-- Lsung: WHERE-Klausel mit Prfung auf NULL-Werte
SELECT
	a.LNr AS 'Nummer Lieferant'
	,LName AS 'Name Lieferant'
	,ANr AS 'Artikelnummer'
	,LMenge AS 'Gelieferte Menge'
	,CONVERT(CHAR(10),LDatum,104) AS 'Lieferdatum'
FROM dbo.Lieferant AS a LEFT OUTER JOIN dbo.Lieferung AS c ON a.LNr = c.LNr
WHERE LMenge IS NULL;

