/*
T-SQL-Abfragen fr Microsoft SQL-Server 2022
Kontakt per E-Mail: info@it-angermann.de
Letzte Aktualisierung:  23 Mrz 2024.
Dirk Angermann
https://it-angermann.de
*/

-- Erstellen einer Sicht "v_HamburgerLief"
CREATE VIEW dbo.v_HamburgerLief
AS 
SELECT 
	LNr AS 'Lieferantennummer'
	,LName AS 'Name des Lieferanten'
	,Status AS 'Status des Lieferanten'
	,LStadt AS 'Firmensitz'
FROM dbo.Lieferant
WHERE LStadt = 'Hamburg';

-- Abfragen der Sicht
SELECT 
	[Lieferantennummer]
	,[Name des Lieferanten]
	,[Status des Lieferanten]
	,[Firmensitz]
FROM dbo.v_HamburgerLief;
go

-- Hinzufgen eines Datensatzes
INSERT INTO dbo.Lieferant VALUES ('L06','Mller',0,'Hamburg');
go

-- Erneutes Abfragen
SELECT 
	[Lieferantennummer]
	,[Name des Lieferanten]
	,[Status des Lieferanten]
	,[Firmensitz]
FROM dbo.v_HamburgerLief;
go

-- Erstellen einer Sicht "v_Sicht1"
CREATE VIEW dbo.v_Sicht1
AS
SELECT
	c.LNr AS 'L_Nummer'
	,LName AS 'L_Name'
	,c.ANr AS 'A_Nummer'
	,AName AS 'A_Name'
	,LMenge AS 'Liefermenge'
	,CONVERT(CHAR(10),LDatum,104) AS 'Lieferdatum'
FROM dbo.Lieferant AS a INNER JOIN dbo.Lieferung AS c ON a.LNr = c.LNr
						INNER JOIN dbo.Artikel AS b ON b.ANr = c.ANr
WHERE LStadt = 'Hamburg'
AND AName IN ('Mutter', 'Schraube')
AND LDatum > '20190801';

-- Abfragen der Sicht
SELECT
	[L_Nummer]
	,[L_Name]
	,[A_Nummer]
	,[A_Name]
	,[Liefermenge]
	,[Lieferdatum]
FROM dbo.v_Sicht1;
go

CREATE FUNCTION dbo.fkt_LiefStadt (@Ort VARCHAR(20))
RETURNS TABLE
AS 
RETURN
(
SELECT 
	LNr
	,LName
	,[Status]
	,LStadt
FROM dbo.Lieferant
WHERE LStadt = @Ort
);
go

SELECT 
	LNr AS 'Nummer'
	,LName AS 'Name'
	,[Status]
	,LStadt AS 'Firmensitz'
FROM dbo.fkt_LiefStadt('Hamburg');
go

SELECT 
	LNr AS 'Nummer'
	,LName AS 'Name'
	,[Status]
	,LStadt AS 'Firmensitz'
FROM dbo.fkt_LiefStadt('Ludwigshafen');
go

SELECT 
	a.ANr AS 'Artikelnummer'
	,a.AName AS 'Artikelname'
FROM (SELECT ANr, AName
	  FROM dbo.Artikel
	  WHERE AMenge BETWEEN 500 and 900) AS a;
go

SELECT 
	Lieferjahr
	,COUNT(LNr) AS 'Anzahl der Lieferungen'
FROM (SELECT
		YEAR(LDatum) AS 'Lieferjahr'
		,LNr
	  FROM dbo.Lieferung) AS Lief 
GROUP BY Lieferjahr;
go

-- Spalten-Alias inline definieren.
SELECT 
	Lieferjahr
	,COUNT(LNr) AS 'Anzahl der Lieferungen'
FROM (SELECT
		YEAR(LDatum) AS 'Lieferjahr'
		,LNr
	   FROM dbo.Lieferung) AS Lief 
GROUP BY Lieferjahr;
go

-- Spalten-Alias extern definieren.
SELECT 
	Lieferjahr
	,COUNT(LNr) AS 'Anzahl der Lieferungen'
FROM (SELECT 
		YEAR(LDatum)
		 ,LNr
	  FROM dbo.Lieferung) AS Lief(Lieferjahr,LNr) 
GROUP BY Lieferjahr;
go

SELECT 
	a.LNr AS 'Nummer'
	,LName AS 'Name'
	,Status
	,LStadt AS 'Firmensitz'
	,COUNT(c.ANr) AS 'Anzahl Bolzen'
FROM dbo.Lieferant AS a INNER JOIN dbo.Lieferung AS c ON a.LNr = c.LNr
						INNER JOIN dbo.Artikel AS b ON b.ANr = c.ANr
WHERE AName ='Bolzen' AND c.LNr <> 'L01'
GROUP BY a.LNr, LName, Status, LStadt
HAVING COUNT(c.ANr) >= (SELECT COUNT(*)
						FROM dbo.Lieferung AS c INNER JOIN dbo.Artikel AS b ON c.ANr = b.ANr
						WHERE AName ='Bolzen' AND c.LNr ='L01');
go

WITH AnzBolzen
AS
(
	SELECT COUNT(b.ANr) AS 'Anzahl Bolzen'
	FROM dbo.Lieferung AS c INNER JOIN dbo.Artikel AS b on c.ANr = b.ANr
	WHERE c.LNr ='L01' AND b.AName ='Bolzen' 
)
SELECT 
	a.LNr AS 'Nummer'
	,LName AS 'Name'
	,Status
	,LStadt AS 'Firmensitz'
	,COUNT(c.ANr) AS 'Anzahl Bolzen'
FROM dbo.Lieferant AS a INNER JOIN dbo.Lieferung AS c ON a.LNr = c.LNr
						INNER JOIN dbo.Artikel AS b ON b.ANr = c.ANr
WHERE AName ='Bolzen' AND c.LNr <> 'L01'
GROUP BY a.LNr, LName, Status, LStadt
HAVING COUNT(c.ANr) >= (SELECT * FROM AnzBolzen);
go


